@@ -45,8 +45,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #define  MEMORYSTEP 8
 
-class _SimpleList:public BaseObj
-{
+class _SimpleList:public BaseObj {
     friend class _AVLList;
 
     protected:
@@ -73,7 +72,7 @@ class _SimpleList:public BaseObj
         _SimpleList(unsigned long);
 
         // stack copy contructor
-        _SimpleList(_SimpleList&,long=0,long=-1);
+        _SimpleList(_SimpleList const&,long=0,long=-1);
 
         // data constructor (1 member list)
         _SimpleList(long);
@@ -103,21 +102,21 @@ class _SimpleList:public BaseObj
         long& operator [] (const long);
 
         // element location functions - read only
-        long operator () (const unsigned long);
+        long operator () (const unsigned long) const;
 
         // assignment operator
-        virtual _SimpleList operator = (_SimpleList);
+        virtual _SimpleList const & operator = (_SimpleList const&);
 
         // append operator
         virtual _SimpleList operator & (_SimpleList);
 
         // append number to this
-        virtual void operator << (long);
+        _SimpleList& operator << (long);
 
         // append number to this if it's not in the list (search first). List assumed unsorted.
         virtual bool operator >> (long);
 
-        virtual void operator << (_SimpleList&);
+        virtual void operator << (_SimpleList const &);
 
 
         /*
@@ -133,7 +132,7 @@ class _SimpleList:public BaseObj
         * @param index The index of the elemnt to retrieve 
         * @return the value of the element at the specified index.
         */
-        long GetElement (const long index);
+        long GetElement (const long index) const;
 
         /**
         * Find the position of a search string in the list of strings (ONLY)
@@ -142,7 +141,7 @@ class _SimpleList:public BaseObj
         * @param startAt Index to start at 
         * @return -1 if not found, index if found
         */
-        virtual long BinaryFind(long, long startAt=0);
+        virtual long BinaryFind(long, long startAt=0) const;
 
         // insert an element into the sorted list preserving the sortedness
         long BinaryInsert(long);
@@ -161,20 +160,27 @@ class _SimpleList:public BaseObj
         * @param j The second index to compare
         * @return -1 if i<j, 0 if i==j, or 1 if i>j 
         */
-        virtual long Compare(long,long);
-        virtual long Compare(BaseRef,long);
+        virtual long Compare(long,long) const;
+        virtual long Compare(BaseObj const*,long) const;
 
-        long CountCommonElements(_SimpleList&, bool=false);
+        long CountCommonElements(_SimpleList const&, bool=false) const;
 
         /**
         //Lists length
         * Example: SimpleList SimpleList([4, 1, 2]).countitems() = 4 
         * @return Unsigned long of item length    
         */
-        unsigned long countitems(void);
+        inline unsigned long countitems(void) const {return lLength;}
 
         /**
-        * SLKP: 20090611    
+         //Is the list empty
+         * Example: SimpleList SimpleList([4, 1, 2]).empty() = false
+         * @return True if the list is empty
+         */
+        bool empty (void) const {return lLength == 0UL;}
+
+        /**
+        * SLKP: 20090611
         * Print the names of variables whose indices are
         * contained in the list
         * @return Nothing. Prints out to screen 
@@ -201,23 +207,71 @@ class _SimpleList:public BaseObj
         */
         virtual void DeleteList(const _SimpleList&);
 
-        // shift a range of elements in the array
+        /** shift a range of elements in the array
+         * Move a block of elements in the list 
+         * Example: SimpleList (1,3,5,7).Displace (1,2,1) = (1,7,3,5)
+         * @param from start the block here (0-based, inclusive)
+         * @param to end the block here (0-based, inclusive)
+         * @param delta number of slots to move (positive or negative, range checked)
+         */
         void Displace(long,long,long);
+        
+        /** Adjust the argument for skipped elements in the [0-max] range
+         * so that the argument is increased by however many elements in the list
+         * are less than it. The list must be sorted
+         * Example: SimpleList (2,4,5).SkipCorrect (2) = 3 
+         * Example: SimpleList (2,4,5).SkipCorrect (4) = 7
+         
+         * @param index the index to correct
+         * @return the corrected index
+         */
+        long SkipCorrect (long) const;
 
+        /** Adjust the argument for skipped elements in the [0-max] range
+         * so that the argument is remapped to the range with elements in this
+         * list excluded. (*this) list must be sorted
+         * Example: SimpleList (2,4,5).SkipCorrect (3) = 2
+         
+         * @param index the index to correct
+         * @param excluded the value to return if index is in the list
+         * @return the corrected index
+         */
+        long CorrectForExclusions (long index, long excluded = -1) const;
+
+        /** Adjust the [sorted] list of indcies argument for skipped elements in the [0-max] range
+         * so that the arguments is remapped to the range with elements in this
+         * list excluded. (*this) list must be sorted
+         * Example: SimpleList (2,4,5).SkipCorrect ([2,3],2) = 1 (and the list is now [2])
+         
+         * @param index the list of indices to correct; corrected indices are written here
+         * @param count the length of indices
+         * @return the number of entires in the indices list that are not excluded
+         */
+  
+        long CorrectForExclusions (long * indices, long count) const;
+  
         /**
         * Much like [] and () except negative indices return offsets from the end. Invalid indices return 0;
         * Example: SimpleList(1,3,5,7).Element(1) = [3]
         * @param index Which item you want.
         * @return A long    
         */
-        long Element(long);
+        long Element(long) const;
 
         /**
-        * Checks if list is identical to other list 
+         * no range checking element access
+         * Example: SimpleList(1,3,5,7).Get (1) = [3]
+         * @param index Which item you want.
+         * @return A long
+         */
+        inline const long Get (long index) const {return lData[index];}
+
+        /**
+        * Checks if list is identical to other list
         * Example: _SimpleList([4, 1, 2]).Equal(_SimpleList([4, 1, 2]) = 4 
         * @return true if equal. 
         */
-        bool Equal(_SimpleList&);
+        bool Equal(_SimpleList const&) const;
 
         /**
         * Retain all those elements that are between (strictly) the 1st and the 2nd argument
@@ -235,7 +289,7 @@ class _SimpleList:public BaseObj
         * @param startAt Index to start at 
         * @return -1 if not found, index if found
         */
-        virtual long Find(long, long startAt = 0);
+        virtual long Find(long, long startAt = 0) const;
 
         /**
         * Same as find, but steps over indices 
@@ -269,7 +323,7 @@ class _SimpleList:public BaseObj
 
         void Intersect(_SimpleList&, _SimpleList&);
 
-        BaseRef ListToPartitionString(void);
+        BaseRef ListToPartitionString(void) const;
 
         virtual BaseRef makeDynamic(void);
 
@@ -279,7 +333,7 @@ class _SimpleList:public BaseObj
         * Example: _SimpleList([4, 1, 2]).Min() = 1 
         * @return maximum value in the list 
         */
-        long Max(void);
+        long Max(void) const;
 
 
         /**
@@ -288,7 +342,7 @@ class _SimpleList:public BaseObj
         * Example: _SimpleList([4, 1, 2]).Sum() = 7 
         * @return the sum of all values in the list 
         */
-        long Sum (void);
+        long Sum (void) const;
 
         /**
         * Populate a Simple List with integers incrementally.
@@ -306,7 +360,7 @@ class _SimpleList:public BaseObj
         * Example: _SimpleList([4, 1, 2]).Min() = 1 
         * @return minimum value in the list 
         */
-        long Min(void);
+        long Min(void) const;
 
 
         /**
@@ -423,7 +477,7 @@ class _SimpleList:public BaseObj
         */
         void Swap(long, long); //swap two elements
 
-        virtual BaseRef toStr(void);
+        virtual BaseRef toStr(unsigned long = 0UL);
 
 
         /**
@@ -449,6 +503,20 @@ class _SimpleList:public BaseObj
         * @return Nothing. Acts on the List object it was called from. 
         */
         void Sort(bool ascending=true);
+  
+        /**
+         * Maps the integer argument to the value of the corresponding 
+         * element in the list. Performs range checks, and returns default value 
+         * if the mapping failed. Equivalent to 
+         * return index >= 0 && index < lLength ? lData[index] : map_failed
+         
+         * @param index the index to map
+         * @param map_failed the value to return if the index is out of range
+         * @return the mapped value
+         */
+  
+         long Map (long index, long map_failed = -1L) const;
+ 
 
 
         /**
@@ -461,9 +529,9 @@ class _SimpleList:public BaseObj
         * the new_order->old_order mapping is returned in the array pointed to
         *
         */
+    
         _SimpleList* CountingSort(long, _SimpleList* = nil);
 
-
         void BubbleSort(void);
         void QuickSort(long, long);
 
