@@ -56,20 +56,22 @@ class _Variable : public _Constant
 public:
 
     _Variable (void);
-    _Variable (_String&, bool isG = false); // name
-    _Variable (_String&, _String&, bool isG = false); // name and formula
+    _Variable (_String const&, bool isG = false); // name
+    _Variable (_String const&, _String const&, bool isG = false); // name and formula
 
     virtual ~_Variable (void);
 
-    virtual   void          Initialize (void);
+    virtual   void          Initialize (bool = false);
     virtual   void          Duplicate (BaseRef);
     virtual   BaseRef       makeDynamic(void);
-    virtual   BaseRef       toStr (void);
-    virtual    void         toFileStr (FILE*);
+    virtual   BaseRef       toStr (unsigned long = 0UL);
+    virtual    void         toFileStr (FILE*, unsigned long = 0UL);
 
     virtual   void          MarkDone (void);
 
     virtual     _PMathObj   Compute (void);       // compute or return the value
+                _PMathObj   ComputeMatchingType (long);
+                // return a value if the type is matched, otherwise nil
     virtual     bool        IsVariable (void); //
     virtual     bool        IsIndependent (void) {
         return (varFormula&&varFormula->theFormula.lLength)?
@@ -78,6 +80,7 @@ public:
     }
     virtual     bool        IsConstant (void);
     void        SetValue (_PMathObj, bool = true); // set the value of the variable
+    void        SetValue (_Parameter); // set the value of the variable
     void        SetNumericValue (_Parameter);
     void        CheckAndSet (_Parameter, bool = false);
     // set the value of the variable
@@ -97,7 +100,7 @@ public:
     virtual     bool        IsCategory (void) {
         return false;
     }
-    virtual     long        GetAVariable (void) {
+    virtual     long        GetAVariable (void) const{
         return theIndex;
     }
     virtual unsigned long        ObjectClass (void) {
@@ -109,7 +112,7 @@ public:
     long        GetIndex (void) {
         return theIndex;
     }
-    virtual     void        ScanForVariables (_AVLList& l, bool globals = false, _AVLListX* tagger = nil, long weight = 0);
+    void        ScanForVariables (_AVLList& l, bool globals = false, _AVLListX* tagger = nil, long weight = 0);
     virtual     bool        IsContainer (void) {
         return false;
     }
@@ -128,19 +131,21 @@ public:
 
     virtual     void        ClearConstraints    (void);
     virtual     bool        CheckFForDependence (long, bool = false);
+    virtual     bool        HasBeenInitialized (void) const {return !(varFlags & HY_VARIABLE_NOTSET);}
+    virtual     void        MarkModified  (void) {varFlags = varFlags | HY_VARIABLE_CHANGED;}
 
-    _String     ContextFreeName                 (void);
-    _String     ParentObjectName                 (void);
+    _String const     ContextFreeName                 (void) const;
+    _String const    ParentObjectName                 (void) const;
  
-    _String*    GetName                         (void) {
+    _String*    GetName                         (void) const{
         return theName;
     }
     _String*    GetFormulaString        (void) {
-        return varFormula?(_String*)varFormula->toStr():(_String*)empty.makeDynamic();
+        return varFormula?(_String*)varFormula->toStr():new _String;
     }
 
     virtual     void        CompileListOfDependents (_SimpleList&);
-    virtual     _PMathObj   ComputeReference        (_PMathObj);
+    _PMathObj   ComputeReference        (_MathObject const *) const;
 
 
     friend      void        ResetVariables          (void);
@@ -156,7 +161,7 @@ public:
     long       theIndex; // index of this variable in the global variable pool
 
     // the class of this variable - i.e global, local, category or random
-    char       varFlags;
+    int       varFlags;
 
     _Parameter lowerBound,
                upperBound;
@@ -166,8 +171,9 @@ public:
 
 };
 
-long    DereferenceVariable (long index, _PMathObj context, char reference_type);
-long    DereferenceString   (_PMathObj, _PMathObj context, char reference_type);
+long    DereferenceVariable (long index, _MathObject const *  context, char reference_type);
+long    DereferenceString   (_PMathObj, _MathObject const * context, char reference_type);
+_String const WrapInNamespace (_String const&, _String const*);
 
 
 #endif
