@@ -4,9 +4,9 @@
  
  Copyright (C) 1997-now
  Core Developers:
- Sergei L Kosakovsky Pond (spond@ucsd.edu)
+ Sergei L Kosakovsky Pond (sergeilkp@icloud.com)
  Art FY Poon    (apoon@cfenet.ubc.ca)
- Steven Weaver (sweaver@ucsd.edu)
+ Steven Weaver (sweaver@temple.edu)
  
  Module Developers:
  Lance Hepler (nlhepler@gmail.com)
@@ -73,7 +73,7 @@ _AVLList        *lookAside = nil;
 _AVLListX       variableNames (&varNamesSupportList),
                 _hyApplicationGlobals (&globalNamesSupportList);
 
-_Parameter printDigits;
+long            printDigits;
 
 
 
@@ -108,13 +108,8 @@ long_max = (_Parameter)LONG_MAX;
  
 //Used in formula, and constant
 
-#ifndef  __HYALTIVEC__
 _Parameter  machineEps = 2.*DBL_EPSILON,
-tolerance  = DBL_EPSILON;
-#else
-_Parameter  machineEps = 1e-7,
-tolerance  = FLT_EPSILON;
-#endif
+            tolerance  = DBL_EPSILON;
 
 //Used in formula
 _String         intPrecFact ("INTEGRATION_PRECISION_FACTOR"),
@@ -136,8 +131,7 @@ _Parameter twoOverSqrtPi = 2./sqrtPi;
 void            DeleteTreeVariable      (long, _SimpleList &, bool);
 
 //__________________________________________________________________________________
-_Variable * LocateVar (long index)
-{
+_Variable * LocateVar (long index) {
     return (_Variable *)(((BaseRef*)variablePtrs.lData)[index]);
 }
 
@@ -220,7 +214,7 @@ _String FetchObjectNameFromType (const unsigned long objectClass) {
             return "Any HyPhy object";
     }
     
-    return empty;
+    return emptyString;
 }
 
 //__________________________________________________________________________________
@@ -246,8 +240,24 @@ _String*   FetchMathObjectNameOfTypeByIndex (const unsigned long objectClass, co
 }
 
 //__________________________________________________________________________________
-_PMathObj   FetchObjectFromVariableByType (_String* id, const unsigned long objectClass, long command_id, _String *errMsg)
-{
+_PMathObj   FetchObjectFromFormulaByType (_Formula& f, const unsigned long objectClass, long command_id, _String *errMsg) {
+  _PMathObj v = f.Compute();
+  if (v) {
+    if (objectClass == HY_ANY_OBJECT || v->ObjectClass () == objectClass) {
+      return v;
+    }
+    if (command_id >= 0 || errMsg) {
+      if (command_id >= 0) {
+        WarnError (_String ("'") & _String ((_String*)f.toStr()) & ("' must evaluate to a ") & FetchObjectNameFromType (objectClass) & " in call to "
+                   &_HY_ValidHBLExpressions.RetrieveKeyByPayload(command_id) & '.');
+      }
+    }
+  }
+  return nil;
+}
+
+//__________________________________________________________________________________
+_PMathObj   FetchObjectFromVariableByType (_String const* id, const unsigned long objectClass, long command_id, _String *errMsg) {
     if (id) {
         _Variable * v = FetchVar (LocateVarByName (*id));
         if (v && (objectClass == HY_ANY_OBJECT || v->ObjectClass () == objectClass)) {
@@ -267,33 +277,32 @@ _PMathObj   FetchObjectFromVariableByType (_String* id, const unsigned long obje
 
 
 //__________________________________________________________________________________
-_PMathObj   FetchObjectFromVariableByTypeIndex (long idx, const unsigned long objectClass, long command_id, _String *errMsg)
-{
+_PMathObj   FetchObjectFromVariableByTypeIndex (long idx, const unsigned long objectClass, long command_id, _String *errMsg) {
     _Variable * v = FetchVar (idx);
-    if (v && (objectClass == HY_ANY_OBJECT || v->ObjectClass () == objectClass)) {
-        return v->GetValue();
-    }
-    if (command_id >= 0 || errMsg) {
-        if (command_id >= 0) {
-            WarnError (_String ("'") & *v->GetName() & ("' must refer to a ") & FetchObjectNameFromType (objectClass) & " in call to " 
-                                     &_HY_ValidHBLExpressions.RetrieveKeyByPayload(command_id) & '.');
-        } else {
-            WarnError (errMsg->Replace ("_VAR_NAME_ID_", *v->GetName(), true));
+    if (v) {
+        if (objectClass == HY_ANY_OBJECT || v->ObjectClass () == objectClass) {
+            return v->Compute ();
         }
-    }    
+        if (command_id >= 0 || errMsg) {
+            if (command_id >= 0) {
+                WarnError (_String ("'") & *v->GetName() & ("' must refer to a ") & FetchObjectNameFromType (objectClass) & " in call to " 
+                                         &_HY_ValidHBLExpressions.RetrieveKeyByPayload(command_id) & '.');
+            } else {
+                WarnError (errMsg->Replace ("_VAR_NAME_ID_", *v->GetName(), true));
+            }
+        }
+    }
     return nil;
 }
 
 //__________________________________________________________________________________
-long LocateVarByName (_String& name)
-{
+long LocateVarByName (_String const& name) {
     return variableNames.Find (&name);
 }
 
 //__________________________________________________________________________________
-_Variable* FetchVar (long index)
-{
-    return index>=0?(_Variable *)variablePtrs(variableNames.GetXtra(index)):nil;
+_Variable* FetchVar (long index) {
+    return index>=0?(_Variable *)variablePtrs.GetItemRangeCheck(variableNames.GetXtra(index)):nil;
 }
 
 //__________________________________________________________________________________
@@ -306,31 +315,17 @@ void       UpdateChangingFlas (long vN)
 
     _SimpleList * toDelete = nil;
 
-    for (long k = 0; k<topLimit; k++) {
+    for (long k = 0L; k<topLimit; k++) {
         long g = ((_SimpleList*)compiledFormulaeParameters.lData[k])->BinaryFind (vN,0);
 
         if (g>=0) {
-            _ElementaryCommand* thisCommand = (_ElementaryCommand*)listOfCompiledFormulae.lData[k];
-            _Formula  *f  = (_Formula*)(thisCommand->simpleParameters.lData[1]),
-                       *f2 = (_Formula*)(thisCommand->simpleParameters.lData[2]);
-
-            delete f;
-            delete f2;
-
-            thisCommand->simpleParameters.Clear();
-
-            // MOD 10/21/2005
+            ((_ElementaryCommand*)listOfCompiledFormulae.lData[k])->DecompileFormulae();
+          
 
             if (!toDelete) {
                 checkPointer(toDelete = new _SimpleList);
             }
 
-            //listOfCompiledFormulae.    Delete(k);
-            //compiledFormulaeParameters.Delete(k);
-
-            //k--;
-            //topLimit--;
-
             *toDelete << k;
         }
     }
@@ -353,15 +348,7 @@ void       UpdateChangingFlas (_SimpleList & involvedVariables)
         long g = ((_SimpleList*)compiledFormulaeParameters.lData[k])->CountCommonElements (involvedVariables,true);
 
         if (g>0) {
-            _ElementaryCommand* thisCommand = (_ElementaryCommand*)listOfCompiledFormulae.lData[k];
-
-            _Formula  *f  = (_Formula*)(thisCommand->simpleParameters.lData[1]),
-                       *f2 = (_Formula*)(thisCommand->simpleParameters.lData[2]);
-
-            delete f;
-            delete f2;
-
-            thisCommand->simpleParameters.Clear();
+            ((_ElementaryCommand*)listOfCompiledFormulae.lData[k])->DecompileFormulae();
 
             if (!toDelete) {
                 checkPointer(toDelete = new _SimpleList);
@@ -413,8 +400,7 @@ void DeleteVariable (long dv, bool deleteself)
                 }
             }
 
-            _Variable* delvar = (FetchVar(dv));
-            DeleteObject (delvar);
+            DeleteObject (FetchVar(dv));
 
             variableNames.Delete (variableNames.Retrieve(dv),true);
             (*((_SimpleList*)&variablePtrs))[vidx]=0;
@@ -525,8 +511,7 @@ void DeleteTreeVariable (long dv, _SimpleList & parms, bool doDeps)
     }
 }
 //__________________________________________________________________________________
-void DeleteVariable (_String&name, bool deleteself)
-{
+void DeleteVariable (_String const &name, bool deleteself) {
     DeleteVariable(LocateVarByName (name), deleteself);
 }
 
@@ -537,7 +522,7 @@ void DeleteTreeVariable (_String&name, _SimpleList& parms, bool doDeps)
 }
 
 //__________________________________________________________________________________
-_Variable* CheckReceptacle (_String* name, _String fID, bool checkValid, bool isGlobal)
+_Variable* CheckReceptacle (_String const * name, _String const & fID, bool checkValid, bool isGlobal)
 {
     if (checkValid && (!name->IsValidIdentifier())) {
         _String errMsg = *name & " is not a valid variable identifier in call to " & fID;
@@ -546,15 +531,17 @@ _Variable* CheckReceptacle (_String* name, _String fID, bool checkValid, bool is
     }
 
     long    f = LocateVarByName (*name);
-    if (f<0) {
+    if ( f<0L ) {
         _Variable dummy (*name, isGlobal);
         f = LocateVarByName (*name);
     }
 
     return FetchVar(f);
 }
+
+
 //__________________________________________________________________________________
-_Variable* CheckReceptacleCommandID (_String* name, const long id, bool checkValid, bool isGlobal, _ExecutionList* context)
+_Variable* CheckReceptacleCommandID (_String const* name, const long id, bool checkValid, bool isGlobal, _ExecutionList* context)
 {
     if (checkValid && (!name->IsValidIdentifier())) {
         _String errMsg = _String ("'") & *name & "' is not a valid variable identifier in call to " & _HY_ValidHBLExpressions.RetrieveKeyByPayload(id) & '.';
@@ -576,7 +563,7 @@ _Variable* CheckReceptacleCommandID (_String* name, const long id, bool checkVal
 }
 
 //__________________________________________________________________________________
-bool CheckReceptacleCommandIDAndStore (_String* name, const long id, bool checkValid, _PMathObj v, bool dup, bool isGlobal)
+bool CheckReceptacleCommandIDAndStore (_String const* name, const long id, bool checkValid, _PMathObj v, bool dup, bool isGlobal)
 {
     _Variable *theV = CheckReceptacleCommandID (name, id, checkValid, isGlobal);
     if (theV) {
@@ -591,8 +578,7 @@ bool CheckReceptacleCommandIDAndStore (_String* name, const long id, bool checkV
 
 
 //__________________________________________________________________________________
-bool CheckReceptacleAndStore (_String* name, _String fID, bool checkValid, _PMathObj v, bool dup)
-{
+bool CheckReceptacleAndStore (_String const* name, _String fID, bool checkValid, _PMathObj v, bool dup) {
     _Variable * theV = CheckReceptacle(name, fID, checkValid);
     if (theV) {
         theV->SetValue (v, dup);
@@ -615,12 +601,6 @@ void  InsertVar (_Variable* theV)
 {
     long pos = variableNames.Insert (theV->theName);
 
-    /*if (theV->GetName()->Equal (&_String("PS_2")))
-    {
-        printf ("Making...\n");
-    }*/
-
-
     if (pos < 0 && isDefiningATree > 1)
         // automatically fix duplicate autogenerated tree node name
     {
@@ -659,14 +639,12 @@ void  InsertVar (_Variable* theV)
 }
 
 //__________________________________________________________________________________
-_String&  AppendContainerName (_String& inString, _VariableContainer* theP)
-{
+_String const&  AppendContainerName (_String const& inString, _VariableContainer const* theP) {
     return AppendContainerName (inString, theP?theP->GetName():nil);
 }
 
 //__________________________________________________________________________________
-_String&  AppendContainerName (_String& inString, _String* namescp)
-{
+_String const&  AppendContainerName (_String const& inString, _String const* namescp) {
     static _String returnMe;
 
     if (_hyApplicationGlobals.Find (&inString) >= 0) {
@@ -721,8 +699,7 @@ void  RenameVariable (_String* oldName, _String* newName)
 }
 
 //__________________________________________________________________________________
-void  ReplaceVar (_Variable* theV)
-{
+void  ReplaceVar (_Variable* theV) {
     long pos = variableNames.Find (theV->theName);
     if (pos>=0) {
         pos = variableNames.GetXtra(pos);
@@ -733,53 +710,56 @@ void  ReplaceVar (_Variable* theV)
     }
 }
 
-//__________________________________________________________________________________
-void    SetupOperationLists (void)
-{
 
-    _List all_unary_ops ("-",29,
-                         "!",
-                         "+",
-                         "*",
-                         "^",
-                         "&",
-                         "Abs",
-                         "Sin",
-                         "Cos",
-                         "Tan",
-                         "Exp",
-                         "Log",
-                         "Arctan",
-                         "Time",
-                         "Gamma",
-                         "Transpose",
-                         "Sqrt",
-                         "Erf",
-                         "Rows",
-                         "Columns",
-                         "LUDecompose",
-                         "Inverse",
-                         "BranchCount",
-                         "TipCount",
-                         "ZCDF",
-                         "Eigensystem",
-                         "Simplex",
-                         "Type",
-                         "Eval",
-                         "LnGamma"
-                         );
+//__________________________________________________________________________________
+void    SetupOperationLists (void) {
+
+  
+    UnOps  < "-" <
+             "!" <
+             "+" <
+             "*" <
+             "^" <
+             "&" <
+             "Abs" <
+             "Sin" <
+             "Cos" <
+             "Tan" <
+             "Exp" <
+             "Log" <
+             "Arctan" <
+             "Time" <
+             "Gamma" <
+             "Transpose" <
+             "Sqrt" <
+             "Erf" <
+             "Rows" <
+             "Columns" <
+             "LUDecompose" <
+             "Inverse" <
+             "BranchCount" <
+             "TipCount" <
+             "ZCDF" <
+             "Eigensystem" <
+             "Simplex" <
+             "Type" <
+             "Eval" <
+             "LnGamma";
  
 
-    UnOps.Insert (all_unary_ops);
-        
     BinOps<<'|'*256+'|';
     opPrecedence<<1;
+    associativeOps << opPrecedence.lLength;
+
     BinOps<<'&'*256+'&';
     opPrecedence<<2;
+    associativeOps << opPrecedence.lLength;
+
     BinOps<<'='*256+'=';
     opPrecedence<<3;
     BinOps<<'!'*256+'=';
     opPrecedence<<3;
+  
     BinOps<<'<';
     opPrecedence<<4;
     BinOps<<'>';
@@ -788,6 +768,7 @@ void    SetupOperationLists (void)
     opPrecedence<<4;
     BinOps<<'>'*256+'=';
     opPrecedence<<4;
+  
     BinOps<<'+';
     associativeOps << opPrecedence.lLength;
     opPrecedence<<5;
@@ -795,6 +776,7 @@ void    SetupOperationLists (void)
     opPrecedence<<5;
     BinOps<<'*';
     associativeOps << opPrecedence.lLength;
+  
     opPrecedence<<6;
     BinOps<<'/';
     opPrecedence<<6;
@@ -802,8 +784,10 @@ void    SetupOperationLists (void)
     opPrecedence<<6;
     BinOps<<'$';
     opPrecedence<<6;
+  
     BinOps<<'^';
     opPrecedence<<7;
+  
     BinOps<<'+'*256+'=';
     opPrecedence<<8;
 
@@ -904,6 +888,7 @@ void    SetupOperationLists (void)
         BuiltInFunctions.AppendNewInstance (new _String ("BranchName"));
         FunctionNameList << BuiltInFunctions (HY_OP_CODE_BRANCHNAME);
         FunctionArgumentCount << 2;
+      
 
         //HY_OP_CODE_CCHI2
         BuiltInFunctions.AppendNewInstance (new _String ("CChi2"));
@@ -915,6 +900,11 @@ void    SetupOperationLists (void)
         FunctionNameList << BuiltInFunctions (HY_OP_CODE_CGAMMADIST);
         FunctionArgumentCount << 3;
 
+        //HY_OP_CODE_CALL
+        BuiltInFunctions.AppendNewInstance (new _String ("Call"));
+        FunctionNameList << BuiltInFunctions (HY_OP_CODE_CALL);
+        FunctionArgumentCount << 2;
+
         //HY_OP_CODE_COLUMNS
         BuiltInFunctions.AppendNewInstance (new _String ("Columns"));
 
@@ -1000,6 +990,8 @@ void    SetupOperationLists (void)
 
         //HY_OP_CODE_MCOORD
         BuiltInFunctions.AppendNewInstance (new _String ("MCoord"));
+        simpleOperationCodes<<HY_OP_CODE_MCOORD;
+        simpleOperationFunctions<<(long)FastMxWrite;
 
         //HY_OP_CODE_MAX
         BuiltInFunctions.AppendNewInstance (new _String ("Max"));
@@ -1038,6 +1030,11 @@ void    SetupOperationLists (void)
         //HY_OP_CODE_SIMPLEX
         BuiltInFunctions.AppendNewInstance (new _String ("Simplex"));
 
+        //HY_OP_CODE_EXPRESSION
+        BuiltInFunctions.AppendNewInstance (new _String ("Simplify"));
+        FunctionNameList << BuiltInFunctions (HY_OP_CODE_SIMPLIFY);
+        FunctionArgumentCount << 2;
+
         //HY_OP_CODE_SIN
         BuiltInFunctions.AppendNewInstance (new _String ("Sin"));
 
@@ -1107,7 +1104,7 @@ void    CompileListOfUserExpressions (_SimpleList& varRefs,_List& rec, bool doAl
         _AVLList sA (&startVars);
         if (doAll) {
 
-            firstVar->ScanForVariables (sA,sA);
+            firstVar->ScanContainerForVariables (sA,sA);
             firstVar->ScanForGVariables (sA,sA);
         }
 
@@ -1146,48 +1143,6 @@ void    CompileListOfUserExpressions (_SimpleList& varRefs,_List& rec, bool doAl
 
 }
 
-//__________________________________________________________________________________
-
-void  FindUnusedObjectName (_String& prefix, _String& partName, _List& names, bool sorted)
-{
-    if (partName.sLength==0) {
-        partName = prefix;
-    }
-
-    _String tryName (partName);
-    long    k = 1;
-
-    if (sorted)
-        while (names.BinaryFind(&tryName)>=0) {
-            k++;
-            tryName = partName&k;
-        }
-    else
-        while (names.Find(&tryName)>=0) {
-            k++;
-            tryName = partName&k;
-        }
-
-    partName = tryName;
-}
-//__________________________________________________________________________________
-
-void  FindUnusedObjectName (_String& prefix, _String& partName, _AVLListX& names, bool)
-{
-    if (partName.sLength==0) {
-        partName = prefix;
-    }
-
-    _String tryName (partName);
-    long    k = 1;
-
-    while (names.Find(&tryName)>=0) {
-        k++;
-        tryName = partName&k;
-    }
-
-    partName = tryName;
-}
 
 //__________________________________________________________________________________
 
