@@ -4,9 +4,9 @@
  
  Copyright (C) 1997-now
  Core Developers:
- Sergei L Kosakovsky Pond (spond@ucsd.edu)
+ Sergei L Kosakovsky Pond (sergeilkp@icloud.com)
  Art FY Poon    (apoon@cfenet.ubc.ca)
- Steven Weaver (sweaver@ucsd.edu)
+ Steven Weaver (sweaver@temple.edu)
  
  Module Developers:
  Lance Hepler (nlhepler@gmail.com)
@@ -46,6 +46,9 @@
 #include "polynoml.h"
 #include "batchlan.h"
 
+#include "function_templates.h"
+
+
 extern _SimpleList freeSlots;
 extern _SimpleList deferIsConstant;
 
@@ -62,7 +65,7 @@ _Variable::_Variable (void)
 }
 
 //__________________________________________________________________________________
-void _Variable::Initialize (void)
+void _Variable::Initialize (bool)
 {
     //_Formula::Initialize();
     _Constant::Initialize();
@@ -125,38 +128,37 @@ bool _Variable::CheckFForDependence (long idx, bool opt)
 }
 
 //__________________________________________________________________________________
-BaseRef _Variable::toStr(void)
+BaseRef _Variable::toStr(unsigned long padding)
 {
     if (varValue&&varValue->IsPrintable()) {
-        return varValue->toStr();
+        return varValue->toStr(padding);
     }
     _PMathObj vv = Compute();
     if (!vv) {
         return new _String("NAN");
     }
-    return new _String((_String*)vv->toStr());
+    return new _String((_String*)vv->toStr(padding));
 }
 
 //__________________________________________________________________________________
-void _Variable::toFileStr(FILE* f)
+void _Variable::toFileStr(FILE* f, unsigned long padding)
 {
     if (varValue&&varValue->IsPrintable()) {
-        varValue->toFileStr(f);
+        varValue->toFileStr(f, padding);
     } else {
         _PMathObj vv = Compute();
         if (!vv) {
             fprintf(f,"NAN");
         } else {
-            vv->toFileStr(f);
+            vv->toFileStr(f, padding);
         }
     }
 
 }
 //__________________________________________________________________________________
 
-_Variable::_Variable (_String&s, bool isG)
-{
-    theName         = (_String*)checkPointer(new _String(s));
+_Variable::_Variable (_String const&s, bool isG) {
+    theName         = new _String(s);
     varFlags        = HY_VARIABLE_NOTSET|(isG?HY_VARIABLE_GLOBAL:0);
     varValue        = nil;
     varFormula      = nil;
@@ -166,11 +168,11 @@ _Variable::_Variable (_String&s, bool isG)
 
 //__________________________________________________________________________________
 
-_Variable::_Variable (_String&s, _String&f, bool isG)//:  _Formula (f)
+_Variable::_Variable (_String const&s, _String const &f, bool isG)//:  _Formula (f)
 {
     //hasBeenChanged = false;
     //isGlobal = isG;
-    theName     = (_String*)checkPointer(new _String(s));
+    theName     = new _String(s);
     varFlags    = isG?HY_VARIABLE_GLOBAL:0;
     varValue    = nil;
     SetBounds   (DEFAULTLOWERBOUND, DEFAULTUPPERBOUND);
@@ -194,7 +196,7 @@ _Variable::_Variable (_String&s, _String&f, bool isG)//:  _Formula (f)
 
 _Variable::~_Variable (void)
 {
-    nInstances++;
+  //nInstances++;
     if (varValue) {
         DeleteObject (varValue);
     }
@@ -222,14 +224,34 @@ void        _Variable::ScanForVariables (_AVLList& l, bool globals, _AVLListX* t
         varFormula->ScanFForVariables(l,globals, false, true, false,tagger, weight);
     }
 }
+  // long call_count = 0L;
+
+  //__________________________________________________________________________________
+
+_PMathObj  _Variable::ComputeMatchingType(long type) {
+  _PMathObj computed_value = Compute();
+  if (computed_value && (computed_value->ObjectClass() & type) > 0L) {
+    return computed_value;
+  }
+  return nil;
+}
 
 
 //__________________________________________________________________________________
 
 _PMathObj  _Variable::Compute (void) // compute or return the value
 {
+    // call_count++;
+  
+    if (varFlags & HY_VARIABLE_COMPUTING) {
+      FlagError (_String ("A recursive dependency error in _Variable::Compute; this is an HBL implementation bug; offending variable is '") & *GetName() & "'");
+    }
+  
+    varFlags |= HY_VARIABLE_COMPUTING;
+  
     if (varFormula == nil) { // no formula, just return the value
         if (varValue) {
+            varFlags &= HY_VARIABLE_COMPUTING_CLR;
             return varValue->Compute();
         }
 
@@ -238,24 +260,34 @@ _PMathObj  _Variable::Compute (void) // compute or return the value
         }
 
         varValue =  new _Constant(theValue);
+        
+        //printf ("Recomputing value of %s => %g\n", theName->sData, varValue->Value());
+
     } else {
-        //printf ("Recomputing value of %s\n", theName->sData);
         if (useGlobalUpdateFlag) {
             if ((varFlags & HY_DEP_V_COMPUTED) && varValue) {
+                varFlags &= HY_VARIABLE_COMPUTING_CLR;
                 return varValue;
             } else if (varFormula->HasChanged()||!varValue) {
+                _PMathObj new_value = (_PMathObj)varFormula->Compute()->makeDynamic();
                 DeleteObject (varValue);
-                varValue = (_PMathObj)varFormula->Compute()->makeDynamic();
+                varValue = new_value;
+                //printf ("Recomputing value of %s => %g\n", theName->sData, varValue->Value());
             }
-
             varFlags |= HY_DEP_V_COMPUTED;
+          
         } else if (varFormula->HasChanged()||!varValue) {
+              /* if varFormula depends on *this* variable, doing delete-set
+                 would cause the expression to reference deleted memory.
+               */
+            _PMathObj new_value = (_PMathObj)varFormula->Compute()->makeDynamic();
             DeleteObject (varValue);
-            varValue = (_PMathObj)varFormula->Compute()->makeDynamic();
+            varValue = new_value;
         }
 
     }
 
+    varFlags &= HY_VARIABLE_COMPUTING_CLR;
     return varValue;
 }
 
@@ -280,16 +312,33 @@ void  _Variable::CompileListOfDependents (_SimpleList& rec)
     }
 }
 
+//__________________________________________________________________________________
+void  _Variable::SetValue (_Parameter new_value) {
+// set the value of the var
+  this->SetValue (new _Constant (new_value), false);
+}
+
 //__________________________________________________________________________________
 void  _Variable::SetValue (_PMathObj theP, bool dup) // set the value of the var
 {
     //hasBeenChanged = true;
+    if (varFlags & HY_VARIABLE_COMPUTING) {
+        FlagError (_String ("A recursive dependency error in _Variable::SetValue; this is an HBL implementation bug; offending variable is '") & *GetName() & "'");
+    }
+  
     varFlags &= HY_VARIABLE_SET;
     varFlags |= HY_VARIABLE_CHANGED;
 
     long     valueClass = theP->ObjectClass();
+  
+    /*bool     doPrint = (*theName) == _String("_value_");
+    if (doPrint) {
+      printf ("Setting %s to %s\n", theName->sData, _String((_String*)theP->toStr()).sData);
+    }*/
 
     if (valueClass==NUMBER) {
+      
+      
         if (varFormula) {
 
             // also update the fact that this variable is no longer dependent in all declared
@@ -313,7 +362,7 @@ void  _Variable::SetValue (_PMathObj theP, bool dup) // set the value of the var
                 }
 
             //_Formula::Clear();
-            delete (varFormula);
+            delete varFormula;
             varFormula = nil;
         }
         if (varValue) {
@@ -336,6 +385,9 @@ void  _Variable::SetValue (_PMathObj theP, bool dup) // set the value of the var
         }
     } else {
         if (varFormula) {
+            /*if (doPrint) {
+              printf ("Resetting constraint on %s which was %s\n", theName->sData,_String((_String*)varFormula->toStr()).sData);
+            }*/
             delete (varFormula);
             varFormula = nil;
             //theFormula.Clear();
@@ -344,6 +396,7 @@ void  _Variable::SetValue (_PMathObj theP, bool dup) // set the value of the var
             DeleteObject (varValue);
             varValue=nil;
         }
+      
         if (valueClass==TREE) {
             variablePtrs.lData[theIndex] = (long)(((_TheTree*)theP)->makeDynamicCopy(GetName()));
             DeleteObject(this);
@@ -361,6 +414,10 @@ void  _Variable::SetValue (_PMathObj theP, bool dup) // set the value of the var
 void  _Variable::SetNumericValue (_Parameter v) // set the value of the var to a number
 {
     //hasBeenChanged = true;
+    if (varFlags & HY_VARIABLE_COMPUTING) {
+      FlagError (_String ("A recursive dependency error in _Variable::SetNumericValue; this is an HBL implementation bug; offending variable is '") & *GetName() & "'");
+    }
+
     varFlags &= HY_VARIABLE_SET;
     varFlags |= HY_VARIABLE_CHANGED;
     theValue = v;
@@ -461,45 +518,53 @@ bool _Variable::IsConstant (void)
         return varFormula->IsConstant();
     }
 
-    if (varValue && varValue->ObjectClass () != NUMBER) {
-        return varValue->IsConstant();
+    if (varValue) {
+        long varType = varValue->ObjectClass ();
+        if (!(varType & HY_MUTABLE_OBJECT)) {
+            return varValue->IsConstant();
+        }
     }
 
     return false;
 }
 
+  //long wp_count = 0;
+
 //__________________________________________________________________________________
 
-void  _Variable::SetFormula (_Formula& theF) // set the value of the var to a formula
-{
+void  _Variable::SetFormula (_Formula& theF) {
+//  bind the variable to an expression
+    if (varFlags & HY_VARIABLE_COMPUTING) {
+      FlagError (_String ("A recursive dependency error in _Variable::SetFormula; this is an HBL implementation bug; offending variable name is '") & *GetName() & "'");
+    }
+
     bool changeMe    = false,
          isAConstant = theF.IsAConstant();
 
-    _Formula* myF = &theF;
+    //if (doPrint) {
+    //printf ("Constraining %s to %s\n", theName->sData, _String((_String*)theF.toStr()).sData);
+    //}
+
+    _Formula* right_hand_side = &theF;
 
     if (isAConstant) {
         _PMathObj theP = theF.Compute();
         if (theP) {
-            myF = new _Formula ((_PMathObj)theP->makeDynamic(),false);
-            checkPointer (myF);
+            right_hand_side = new _Formula ((_PMathObj)theP->makeDynamic(),false);
         } else {
             return;
         }
     }
 
     _SimpleList vars;
-    {
-        _AVLList vA (&vars);
-        theF.ScanFForVariables (vA,true);
-        vA.ReorderList();
-    }
+    _AVLList vA (&vars);
+    theF.ScanFForVariables (vA,true);
+    vA.ReorderList();
 
     if (vars.BinaryFind(theIndex)>=0) {
-        _String * sf = (_String*)theF.toStr();
-        WarnError ((_String("Can't set variable ")&*GetName()&" to "&*sf&" because it would create a circular dependance."));
-        DeleteObject(sf);
-        if (&theF!=myF) {
-            delete myF;
+        WarnError ((_String("Can't set variable ")&*GetName()&" to "&*((_String*)theF.toStr())&" because it would create a circular dependance."));
+        if (&theF!=right_hand_side) {
+            delete right_hand_side;
         }
         return;
     }
@@ -510,7 +575,10 @@ void  _Variable::SetFormula (_Formula& theF) // set the value of the var to a fo
         varFlags -= HY_VARIABLE_CHANGED;
     }
 
-
+    if (varFlags & HY_DEP_V_COMPUTED) {
+        varFlags -= HY_DEP_V_COMPUTED;
+    }
+    
     if (varFormula) {
         delete (varFormula);
         varFormula = nil;
@@ -519,50 +587,50 @@ void  _Variable::SetFormula (_Formula& theF) // set the value of the var to a fo
     }
 
     if (varValue) {
-        DeleteObject (varValue);
-        varValue=nil;
+        DeleteAndZeroObject (varValue);
     }
 
     //_Formula::Duplicate ((BaseRef)myF);
     varFormula = new _Formula;
-    varFormula->Duplicate ((BaseRef)myF);
+    varFormula->Duplicate ((BaseRef)right_hand_side);
 
     // mod 20060125 added a call to simplify constants
     varFormula->SimplifyConstants ();
 
     // also update the fact that this variable is no longer independent in all declared
-    // variable containers which contain references to this variable
-    if (changeMe)
-        if (deferSetFormula) {
-            *deferSetFormula << theIndex;
-            deferIsConstant  << isAConstant;
-        } else {
-            long i;
-            _SimpleList tcache;
-            long        iv;
+    // variable containers which hold references to this variable
+    if (changeMe) {
+          if (deferSetFormula) {
+              *deferSetFormula << theIndex;
+              deferIsConstant  << isAConstant;
+          } else {
+              long i;
+              _SimpleList tcache;
+              long        iv;
 
-            i = variableNames.Traverser (tcache,iv,variableNames.GetRoot());
+              i = variableNames.Traverser (tcache,iv,variableNames.GetRoot());
 
-            for (; i >= 0; i = variableNames.Traverser (tcache,iv)) {
-                _Variable* theV = FetchVar(i);
-                if (theV->IsContainer()) {
-                    _VariableContainer* theVC = (_VariableContainer*)theV;
-                    if (theVC->SetDependance(theIndex) == -2) {
-                        ReportWarning ((_String("Can't make variable ")&*GetName()&" dependent in the context of "&*theVC->GetName()&" because its template variable is bound by another relation in the global context."));
-                        continue;
-                    }
-                }
-            }
-            {
-                for (long i = 0; i<likeFuncList.lLength; i++)
-                    if (((_String*)likeFuncNamesList(i))->sLength) {
-                        ((_LikelihoodFunction*)likeFuncList(i))->UpdateIndependent(theIndex,isAConstant);
-                    }
-            }
-        }
+              for (; i >= 0; i = variableNames.Traverser (tcache,iv)) {
+                  _Variable* theV = FetchVar(i);
+                  if (theV->IsContainer()) {
+                      _VariableContainer* theVC = (_VariableContainer*)theV;
+                      if (theVC->SetDependance(theIndex) == -2) {
+                          ReportWarning ((_String("Can't make variable ")&*GetName()&" dependent in the context of "&*theVC->GetName()&" because its template variable is bound by another relation in the global context."));
+                          continue;
+                      }
+                  }
+              }
+              {
+                  for (unsigned long i = 0UL; i<likeFuncList.lLength; i++)
+                      if (((_String*)likeFuncNamesList(i))->sLength) {
+                          ((_LikelihoodFunction*)likeFuncList(i))->UpdateIndependent(theIndex,isAConstant);
+                      }
+              }
+          }
+    }
 
-    if (&theF!=myF) {
-        delete myF;
+    if (&theF!=right_hand_side) {
+        delete right_hand_side;
     }
 }
 
@@ -584,28 +652,28 @@ void  _Variable::PreMarkChanged (void)
 }
 
 //__________________________________________________________________________________
-void  _Variable::PostMarkChanged (void)
-{
+void  _Variable::PostMarkChanged (void) {
     varFlags &= HY_DEP_CLEAR_MASK;
 }
 
 
 //__________________________________________________________________________________
-bool  _Variable::HasChanged (bool ignoreCats) // does this variable need recomputing
-{
+bool  _Variable::HasChanged (bool ignoreCats) {
+    // does this variable need recomputing 
     if (varFormula) {
         if (useGlobalUpdateFlag && (varFlags&HY_DEP_V_COMPUTED)) {
             return false;
         }
 
-        if (varFlags&HY_DEP_V_INSPECTED) {
+        /*if (varFlags&HY_DEP_V_INSPECTED) {
             return ignoreCats?(varFlags&HY_DEP_V_MODIFIED_CATS):(varFlags&HY_DEP_V_MODIFIED);
-        }
+        }*/
+        // SLKP 20170202: seems to be unused
 
-        return  varFormula->HasChanged(ignoreCats);
+       return  !varValue || varFormula->HasChanged(ignoreCats) ;
 
     } else {
-        if (varValue&&(varValue->IsVariable())) {
+        if (varValue && varValue->IsVariable()) {
             return varValue->HasChanged();
         }
         if (ignoreCats && IsCategory()) {
@@ -618,24 +686,22 @@ bool  _Variable::HasChanged (bool ignoreCats) // does this variable need recompu
 
 //__________________________________________________________________________________
 
-void _Variable::MarkDone (void)
-{
+void _Variable::MarkDone (void) {
     if (!varFormula && (varFlags & HY_VARIABLE_CHANGED) && !(varValue && varValue->IsVariable())) {
         varFlags -= HY_VARIABLE_CHANGED;
     }
 }
 
 //__________________________________________________________________________________
-_PMathObj    _Variable::ComputeReference (_PMathObj context)
-{
+_PMathObj    _Variable::ComputeReference (_MathObject const * context) const {
     _String reference_string (*GetName());
-    reference_string = AppendContainerName(reference_string, (_VariableContainer*)context);
+    reference_string = AppendContainerName(reference_string, (_VariableContainer const*)context);
     
     return new _FString (reference_string, false);
 }
 
 //__________________________________________________________________________________
-_String    _Variable::ContextFreeName(void) {
+_String const    _Variable::ContextFreeName(void) const {
     long location = theName->FindBackwards (".", 0, -1);
     if (location > 0) {
        return theName->Cut (location+1,-1); 
@@ -644,16 +710,23 @@ _String    _Variable::ContextFreeName(void) {
 }
 
 //__________________________________________________________________________________
-_String    _Variable::ParentObjectName(void) {
+_String const   _Variable::ParentObjectName(void) const {
     long location = theName->FindBackwards (".", 0, -1);
     if (location > 0) {
        return theName->Cut (0,location-1); 
     }  
-    return empty;
+    return emptyString;
+}
+
+_String const WrapInNamespace (_String const& name, _String const* context) {
+  if (context) {
+    return *context & '.' & name;
+  }
+  return name;
 }
 
 //__________________________________________________________________________________
-long    DereferenceString (_PMathObj v, _PMathObj context, char reference_type){
+long    DereferenceString (_PMathObj v, _MathObject const * context, char reference_type){
     if (v && v->ObjectClass () == STRING) {
         _FString * value = (_FString*)v;
         _String referencedVariable = *value->theString;
@@ -666,7 +739,7 @@ long    DereferenceString (_PMathObj v, _PMathObj context, char reference_type){
 }
 
 //__________________________________________________________________________________
-long    DereferenceVariable (long index, _PMathObj context, char reference_type){
+long    DereferenceVariable (long index, _MathObject const * context, char reference_type){
     if (reference_type == HY_STRING_DIRECT_REFERENCE) {
         return index;
     }
